(***************************************************)
(* Le module Frame dfinit les conventions d'appel *)
(***************************************************)

open Gen

type frame
(* Le type Frame.frame dcrit les fonctions (sous-routines) du code intermdiaire *)

val named_frame : string -> Pp.var_list -> Pp.type_expr option -> frame
   (* Creation du frame d'une fonction/procdure, dont le nom est
      pass en premier argument, les paramtres formels en second argument,
      et le (type du) rsultat en dernier argument *)
val frame_name : frame -> label
   (* retourne le point d'entre de la sous-routine *)
val frame_args : frame -> temp list
   (* retourne la liste des temporaires choisis pour recevoir les arguments *)
val frame_result : frame -> temp option
   (* Renvoie le temporaire choisi pour retourner
      le rsultat d'une vraie fonction, None pour une procdure *)
val frame_return : frame -> label
   (* Renvoie l'tiquette choisie pour l'pilogue (marque la fin de
   la sous-routine) *)

val frame_size : frame -> int
    (* retourne la taille du frame (nombre de locaux) *)
val frame_size_label : frame -> string
    (* retourne le nom de la constante qui dfinit la taille du frame *)


(**********************************)
(* Allocation en pile (par mots)  *)
(**********************************)    
val alloc_local : frame -> int
   (* au fond du frame, renvoie la position alloue par rapport  sp *)
val make_space_for_args : frame -> int -> unit
   (* au sommet du frame *)

(**********************************)
(* Allocation statique            *)
(**********************************)
val global_space : label
  (* tiquette de la zone (sert deux fois!) *)
val global_register : temp
  (* registre qui point sur la zone *)


(*************************)
(* Frames des primitives *)
(*************************)

val write_int : frame
val writeln_int : frame
val read_int : frame
val alloc  : frame
val bidon : frame (* frame bidon, parfois utile *)

val wordsize : int
    (* sans commentaire *)
